% Why Do Americans Work More that Europeans, Prescott 2004
% prescott.m
% Numerical Methods for Macroeconomists, by Jeremy Greenwood and Ricardo 
% Marto, Copywrite, Jeremy Greenwood

clear all
clc

global alpha theta 
global tau9396 ctoo9396 hdata9396 tau7074 ctoo7074 hdata7074

% Load Data
% Data 1993 - 96
tau9396 = [.59,.59,.64,.52,.44,.37,.40];          % Effective tax rate
ctoo9396 = [.74,.74,.69,.77,.83,.68,.81];         % Cons/output ratio 
hdata9396 = [19.3,17.5,16.5,22.9,22.8,27.0,25.9]; % Hours data

% Data 1970 - 74
tau7074 = [.52,.49,.41,.44,.45,.25,.40];   
ctoo7074 = [.66,.66,.66,.72,.77,.60,.74];
hdata7074 = [24.6,24.4,19.2,22.2,25.9,29.8,23.5];

% Construct character array of names
names = char('Ger', 'Fra', 'Ita','Can', 'UK', 'Jap', 'USA');

% Set Parameter Values
% Parameter values
alpha = 1.54; % Weight on leisure
theta = 0.32; % Capitals share

% Calculate Model's Predictions using Prescott's alpha

% 1993-96

hmodel9396 = hours(tau9396,ctoo9396); % Function for model's solution
error9396  = hdata9396-hmodel9396;    % Prediction error 
disp('Results using Prescotts alpha')
disp('alpha')
disp(alpha)
disp('Results--1993 to 96')
T = table(names, hdata9396', hmodel9396', error9396', ...
    'VariableNames',{'Country' 'Data' 'Model' 'Error'});
disp(T)


% 1970-74
hmodel7074 = hours(tau7074,ctoo7074); % Function for model's solution
error7074  = hdata7074-hmodel7074;    % Prediction error 

disp('Results--1970 to 74')
T = table(names, hdata7074', hmodel7074', error7074', ...
    'VariableNames',{'Country' 'Data' 'Model' 'Error'});
disp(T)

% Compute and display euclidean norm for the prediction error 
% over both periods

disp('norm')
disp( norm( [error9396 error7074] ) )
% norm is MATLAB command for the euclidean norm
% Observe how the predictions errors have been patched together into a
% single vector. 

% Make graphs plotting the data versus model for the 7 countries.
% 1993-96
% Figure 1
heading = 'Fit of Model--Prescotts alpha, 1993 to 1996'; 
% Call function to make the figure
makefigures( 1, hdata9396, hmodel9396, heading)
% 1970-74
% Figure 2
heading = 'Fit of Model--Prescotts alpha, 1970 to 1974'; 
makefigures( 2, hdata7074, hmodel7074, heading)

% Export Data to an excel file

Data9396 = table(names, hdata9396', hmodel9396',hdata9396', ...
    'VariableNames',{'Country' 'Data' 'Model' '45Deg'});
filename = 'Data9396.xlsx';
writetable(Data9396,filename)  
Data7074 = table(names, hdata7074', hmodel7074',hdata7074', ...
    'VariableNames',{'Country' 'Data' 'Model' '45Deg'});
filename = 'Data7074.xlsx';
writetable(Data7074,filename)

% Calculate the optimal value for alpha

% Call nonlinear equation solver to compute least squares problem
alpha = fzero(@foc, alpha);
if abs( foc(alpha) ) >= 0.0000001       % Check foc is close to zero
    disp('Solution not found') 
end

% Calculate Model's Predictions--by using the first-order condition for 
% for the least squares problem

% 1993-96

hmodel9396 = hours(tau9396,ctoo9396);
error9396  = hdata9396-hmodel9396;
disp('Results using the alpha that fits the best')
disp('Two approaches are taken:')
disp('a: solve the first-order condition from least squares problem')
disp('b: solve the least squares minimization problem')
disp('alpha--from first-order condition')
disp(alpha)
disp('Results--1993 to 96')
T = table(names, hdata9396', hmodel9396', error9396', ...
    'VariableNames',{'Country' 'Data' 'Model' 'Error'});
disp(T)

% 1970-74

disp('Results--1970 to 74')
disp('   Data      Model      Error')
hmodel7074 = hours(tau7074,ctoo7074);
error7074  = hdata7074-hmodel7074;
T = table(names, hdata7074', hmodel7074', error7074', ...
    'VariableNames',{'Country' 'Data' 'Model' 'Error'});
disp(T)
disp('norm')
disp ( norm( [ error9396 error7074 ] ) )


% Make graphs to show fit
% 1993-96
% Figure 3
heading = 'Fit of Model--fitted alpha, 1993 to 1996'; 
% Call function to make the figure
makefigures( 3, hdata9396, hmodel9396, heading)
% 1970-74
% Figure 4
heading = 'Fit of Model--fitted alpha, 1970 to 1974'; 
makefigures( 4, hdata7074, hmodel7074, heading)

% Calculate Model's Predictions--by minimizing the sum of the squares

alpha = fminbnd(@sumofsquares, 1,2); % Call up minimization routine
disp('alpha--by minimizing the sum of the squares')
disp(alpha)



